package com.community.pocket.data.resetpwd;

import androidx.lifecycle.MutableLiveData;

import com.community.pocket.ui.resetpwd.ResetPwdResponse;
import com.community.pocket.util.HttpRequest;
import com.community.pocket.util.HttpUtil;
import com.community.pocket.util.SimpleHttpParse;

import okhttp3.FormBody;

/**
 * 重置密码请求
 */
public class ResetPwdRequest {
    private static volatile ResetPwdRequest instance;

    private ResetPwdRequest() {
    }

    public static ResetPwdRequest getInstance() {
        if (instance == null) {
            instance = new ResetPwdRequest();
        }
        return instance;
    }

    /**
     * 检查用户邮箱
     */
    @HttpRequest("/resetpwd/checkUser")
    public void checkUser(MutableLiveData<ResetPwdResponse> liveData, String username, String email) {
        HttpUtil.getRequest(HttpUtil.Method.POST, new SimpleHttpParse<ResetPwdResponse>(liveData).getInterface(ResetPwdResponse.class),
                new FormBody.Builder()
                        .add("username", username)
                        .add("email", email)
                        .build());
    }


    /**
     * 发送验证码
     */
    @HttpRequest("/resetpwd/sendCode")
    public void sendCode(MutableLiveData<ResetPwdResponse> liveData, String email) {
        HttpUtil.getRequest(HttpUtil.Method.POST, new SimpleHttpParse<ResetPwdResponse>(liveData).getInterface(ResetPwdResponse.class),
                new FormBody.Builder()
                        .add("email", email)
                        .build());
    }

    /**
     * 检查验证码
     */
    @HttpRequest("/resetpwd/checkCode")
    public void checkCode(MutableLiveData<ResetPwdResponse> liveData, String email, String code) {
        HttpUtil.getRequest(HttpUtil.Method.POST, new SimpleHttpParse<ResetPwdResponse>(liveData).getInterface(ResetPwdResponse.class),
                new FormBody.Builder()
                        .add("email", email)
                        .add("code", code)
                        .build());
    }

    /**
     * 重置密码
     */
    @HttpRequest("/resetpwd/resetPwd")
    public void resetPwd(MutableLiveData<ResetPwdResponse> liveData, String username, String password) {
        HttpUtil.getRequest(HttpUtil.Method.POST, new SimpleHttpParse<ResetPwdResponse>(liveData).getInterface(ResetPwdResponse.class),
                new FormBody.Builder()
                        .add("username", username)
                        .add("password", password)
                        .build());
    }
}
