package com.community.pocket.ui;

import android.os.Bundle;
import android.text.format.DateFormat;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.community.pocket.R;
import com.community.pocket.ui.main.ui.share.Response;
import com.community.pocket.util.CustomMessage;

import org.xutils.x;

/**
 * 封装初始化xutils逻辑
 */
public abstract class BaseFragment extends Fragment {
    @NonNull
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        //fragment注解
        return x.view().inject(this, inflater, container);
    }

    //获取父级Fragment
    protected <T extends Fragment> T getParentFragment(Class<T> tClass) {
        return getParentFragment(tClass, null);
    }

    //校验token失败跳转到登陆页
    protected <E extends CustomMessage, T extends Response<E>> boolean invalidToken(T t) {
        return t.getMessage() != null && t.getMessage().getMsg() == R.string.invalid_token;
    }

    //获取父级Fragment
    @SuppressWarnings("unchecked")
    private <T extends Fragment> T getParentFragment(Class<T> tClass, Fragment fragment) {
            if (fragment == null) {
                fragment = getParentFragment();
            } else {
                fragment = fragment.getParentFragment();
            }
        if (fragment != null) {
            return fragment.getClass().isAssignableFrom(tClass) ? (T) fragment : getParentFragment(tClass, fragment);
        } else {
            throw new RuntimeException("not fround parent Fragment for " + tClass);
        }

    }

    //格式化日期
    protected String formatUnix(Long time) {
        return DateFormat.format(getString(R.string.dateformat), time).toString();
    }

}
