package com.community.pocket.ui.main.ui.forum;

import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Build;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.appcompat.widget.SearchView;
import androidx.core.widget.NestedScrollView;
import androidx.lifecycle.MutableLiveData;

import com.community.pocket.R;
import com.community.pocket.data.main.info.InfoRequest;
import com.community.pocket.data.model.Forum;
import com.community.pocket.data.model.ForumStatus;
import com.community.pocket.data.model.Page;
import com.community.pocket.ui.BaseFragment;
import com.community.pocket.ui.main.ui.forum.data.ForumDataActivity;
import com.community.pocket.ui.main.ui.forum.main.ForumFragment;
import com.community.pocket.util.Param;

import org.xutils.view.annotation.ViewInject;

public abstract class ForumPost extends BaseFragment {

    //显示发帖人
    protected abstract int showAuthor();

    //显示审核状态
    protected abstract int showStatus();

    @ViewInject(R.id.post_layout)
    private LinearLayout layout;


    @ViewInject(R.id.search)
    private SearchView searchView;


    /**
     * 加载帖子数据
     */
    protected void loadPost(Page<Forum> forumList) {

        if (forumList.getCurrentPage() == 1 && !forumList.getList().isEmpty()) {
            layout.removeAllViews();
        }
            for (int i = 0; i < forumList.getList().size(); i++) {
                final Forum forum = forumList.getList().get(i);

                View childView = View.inflate(getContext(), R.layout.forum_list, null);
                TextView status = childView.findViewById(R.id.check_status);
                status.setVisibility(showStatus());
                switch (forum.getStatus()) {
                    case ok:
                        status.setText(R.string.forum_status_ok);
                        break;
                    case fail:
                        status.setText(R.string.forum_status_fail);
                        break;
                    case uncheck:
                        status.setText(R.string.forum_status_uncheck);
                        break;
                    default:
                }

                TextView title = childView.findViewById(R.id.post_title);
                title.setText(forum.getTitle());
                TextView content = childView.findViewById(R.id.post_content);
                content.setText(forum.getContent());
                TextView author = childView.findViewById(R.id.poster);
                author.setVisibility(showAuthor());
                author.setText(getString(R.string.poster, forum.getUsername()));

                ImageView headimg = childView.findViewById(R.id.headimg);
                MutableLiveData<Bitmap> liveData = new MutableLiveData<>();
                InfoRequest.getInstance().getImg(liveData, forum.getInfo().getHeadImg());
                liveData.observe(getViewLifecycleOwner(), headimg::setImageBitmap);

                TextView postReply = childView.findViewById(R.id.post_reply);
                postReply.setText(getString(R.string.post_reply, forum.getReply()));
                TextView time = childView.findViewById(R.id.post_time);
                time.setText(getString(R.string.post_time, formatUnix(forum.getTime())));
                ViewGroup.MarginLayoutParams layoutParams = new ViewGroup.MarginLayoutParams(new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
                layoutParams.setMargins(0, 0, 0, 50);
                childView.setLayoutParams(layoutParams);


                //打开帖子
                Button button = childView.findViewById(R.id.showAll);
                if (forum.getStatus() == ForumStatus.ok) {
                    button.setOnClickListener(v -> {
                        Intent intent = new Intent(getContext(), ForumDataActivity.class);
                        intent.putExtra(Param.forumId.name(), forum.getId());
                        startActivityForResult(intent, 1);
                    });
                } else {
                    button.setVisibility(View.GONE);
                }

                layout.addView(childView);

            }
    }

    protected abstract FormLoad getViewModel();

    protected abstract Long getCurrentPage();

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);

        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(String query) {
                if (!query.isEmpty()) {
                    load(1L);
                    return true;
                } else {
                    return false;
                }

            }

            @Override
            public boolean onQueryTextChange(String newText) {
                return false;
            }
        });
    }

    protected void load(Long page) {
        getViewModel().loadForumNew(page, searchView.getQuery().toString());
    }

    //滚动加载更多帖子
    @RequiresApi(api = Build.VERSION_CODES.M)
    protected void loadMore() {
        //滚动加载更多帖子
        ForumFragment forumFragment = getParentFragment(ForumFragment.class);
        forumFragment.getScrollView().setOnScrollChangeListener((NestedScrollView.OnScrollChangeListener) (v, scrollX, scrollY, oldScrollX, oldScrollY) -> {
            if (scrollY == (v.getChildAt(0).getMeasuredHeight() - v.getMeasuredHeight())) {
                load(getCurrentPage() + 1);
            }
        });
    }
}
