package com.community.pocket.ui.main.ui.forum.post;

import android.os.Build;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.text.format.DateFormat;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;

import com.community.pocket.R;
import com.community.pocket.ui.listener.MyTextChange;
import com.community.pocket.util.PropertiesUtil;

import org.xutils.view.annotation.ContentView;
import org.xutils.view.annotation.ViewInject;

/**
 * 活动贴
 */
@ContentView(R.layout.forum_post_active_fragment)
public class ForumPostActiveFragment extends ForumPostContent {

    //活动开始时间
    @ViewInject(R.id.active_start_time)
    private EditText activeStartTime;

    //活动结束时间
    @ViewInject(R.id.active_end_time)
    private EditText activeEndTime;

    //活动信用分
    @ViewInject(R.id.active_score)
    private EditText activeScore;

    //报名截止日期
    @ViewInject(R.id.registration_deadline)
    private EditText registrationDeadline;

    private ForumPostActiveViewModel viewModel;

    //日期格式提示
    @ViewInject(R.id.tip)
    private TextView tip;

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);


        viewModel = new ViewModelProvider(this, new ViewModelProvider.NewInstanceFactory()).get(ForumPostActiveViewModel.class);

        //监听表单校验状态
        viewModel.getForumPostFormState().observe(getViewLifecycleOwner(), new Observer<ForumPostFormState.Active>() {
            @Override
            public void onChanged(ForumPostFormState.Active forumPostFormState) {
                if (forumPostFormState == null) {
                    return;
                }

                if (forumPostFormState.getTitleError() != null) {
                    postTitle.setError(getString(forumPostFormState.getTitleError()));
                }
                if (forumPostFormState.getContentError() != null) {
                    postContent.setError(getString(forumPostFormState.getContentError()));
                }
                if (forumPostFormState.getRegistrationDeadlineError() != null) {
                    registrationDeadline.setError(getString(forumPostFormState.getRegistrationDeadlineError()));
                }
                if (forumPostFormState.getActiveStartTimeError() != null) {
                    activeStartTime.setError(getString(forumPostFormState.getActiveStartTimeError()));
                }
                if (forumPostFormState.getActiveEndTimeError() != null) {
                    activeEndTime.setError(getString(forumPostFormState.getActiveEndTimeError()));
                }
                if (forumPostFormState.getActiveScore() != null) {
                    activeScore.setError(getString(forumPostFormState.getActiveScore(), PropertiesUtil.getIntValue("score.max")));
                }

                postButton.setEnabled(forumPostFormState.isDataValid());
            }
        });

        //监听发帖状态
        sendPost(viewModel);

        TextWatcher textWatcher = new MyTextChange() {
            @Override
            public void afterTextChanged(Editable s) {
                viewModel.postDataForumChanged(postTitle.getText().toString(), postContent.getText().toString(), activeStartTime.getText().toString(), activeEndTime.getText().toString(), activeScore.getText().toString(), registrationDeadline.getText().toString());
            }
        };
        postTitle.addTextChangedListener(textWatcher);
        postContent.addTextChangedListener(textWatcher);
        registrationDeadline.addTextChangedListener(textWatcher);
        activeStartTime.addTextChangedListener(textWatcher);
        activeEndTime.addTextChangedListener(textWatcher);
        activeScore.addTextChangedListener(textWatcher);

        //发帖操作
        postButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                viewModel.sendPost(postTitle.getText().toString(), postContent.getText().toString(), activeStartTime.getText().toString(), activeEndTime.getText().toString(), activeScore.getText().toString(), registrationDeadline.getText().toString());
            }
        });

        initTip();
    }

    //初始化日期提示
    private void initTip() {
        String format = PropertiesUtil.getValue("date.pattern");
        tip.setText(getString(R.string.dateformat_tip, format, DateFormat.format(format, System.currentTimeMillis())));
    }

}
