package com.community.pocket.ui.main.ui.forum.post;

import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.widget.EditText;

import androidx.annotation.Nullable;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;

import com.community.pocket.R;
import com.community.pocket.ui.listener.MyTextChange;
import com.community.pocket.util.PropertiesUtil;

import org.xutils.view.annotation.ContentView;
import org.xutils.view.annotation.ViewInject;

/**
 * 悬赏贴布局
 */
@ContentView(R.layout.forum_post_score_fragment)
public class ForumPostScore extends ForumPostContent {

    private ForumPostScoreViewModel viewModel;

    @ViewInject(R.id.active_score)
    private EditText activeScore;

    public static ForumPostScore newInstance() {
        return new ForumPostScore();
    }

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        viewModel = new ViewModelProvider.NewInstanceFactory().create(ForumPostScoreViewModel.class);

        //监听表单校验状态
        viewModel.getForumPostFormState().observe(getViewLifecycleOwner(), new Observer<ForumPostFormState.Score>() {
            @Override
            public void onChanged(ForumPostFormState.Score forumPostFormState) {
                if (forumPostFormState == null) {
                    return;
                }

                if (forumPostFormState.getTitleError() != null) {
                    postTitle.setError(getString(forumPostFormState.getTitleError()));
                }
                if (forumPostFormState.getContentError() != null) {
                    postContent.setError(getString(forumPostFormState.getContentError()));
                }
                if (forumPostFormState.getScoreError() != null) {
                    activeScore.setError(getString(forumPostFormState.getScoreError(), PropertiesUtil.getIntValue("score.max")));
                }

                postButton.setEnabled(forumPostFormState.isDataValid());
            }
        });

        //监听发帖状态
        sendPost(viewModel);

        TextWatcher textWatcher = new MyTextChange() {
            @Override
            public void afterTextChanged(Editable s) {
                viewModel.scoreFormChanged(postTitle.getText().toString(), postContent.getText().toString(), activeScore.getText().toString());
            }
        };
        postTitle.addTextChangedListener(textWatcher);
        postContent.addTextChangedListener(textWatcher);
        activeScore.addTextChangedListener(textWatcher);

        //发帖操作
        postButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                viewModel.sendScore(postTitle.getText().toString(), postContent.getText().toString(), activeScore.getText().toString());
            }
        });
    }

}
