package com.community.pocket.ui.main.ui.forum.post;

import android.os.Build;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;

import com.community.pocket.R;
import com.community.pocket.ui.listener.MyTextChange;

import org.xutils.view.annotation.ContentView;

/**
 * 动态贴
 */
@ContentView(R.layout.forum_post_topic_fragment)
public class ForumPostTopicFragment extends ForumPostContent {

    private ForumPostTopicViewModel viewModel;

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        viewModel = new ViewModelProvider(this, new ViewModelProvider.NewInstanceFactory()).get(ForumPostTopicViewModel.class);

        //监听表单校验状态
        viewModel.getForumPostFormState().observe(getViewLifecycleOwner(), new Observer<ForumPostFormState>() {
            @Override
            public void onChanged(ForumPostFormState forumPostFormState) {
                if (forumPostFormState == null) {
                    return;
                }

                if (forumPostFormState.getTitleError() != null) {
                    postTitle.setError(getString(forumPostFormState.getTitleError()));
                }
                if (forumPostFormState.getContentError() != null) {
                    postContent.setError(getString(forumPostFormState.getContentError()));
                }

                postButton.setEnabled(forumPostFormState.isDataValid());
            }
        });

        //监听发帖状态
        sendPost(viewModel);

        TextWatcher textWatcher = new MyTextChange() {
            @Override
            public void afterTextChanged(Editable s) {
                viewModel.topicFormChanged(postTitle.getText().toString(), postContent.getText().toString());
            }
        };
        postTitle.addTextChangedListener(textWatcher);
        postContent.addTextChangedListener(textWatcher);

        //发帖操作
        postButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                viewModel.sendTopic(postTitle.getText().toString(), postContent.getText().toString());
            }
        });
    }
}
