package com.community.pocket.ui.main.ui.garbage.sorting;

import android.os.Bundle;
import android.text.Editable;
import android.view.View;
import android.widget.AutoCompleteTextView;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.ViewModelProvider;

import com.community.pocket.R;
import com.community.pocket.data.adapter.GarbageSortingAdpter;
import com.community.pocket.data.model.GarbageSorting;
import com.community.pocket.ui.BaseFragment;
import com.community.pocket.ui.listener.MyTextChange;
import com.community.pocket.ui.main.ui.share.MyAutoCompleteTextView;
import com.community.pocket.ui.main.ui.share.Response;
import com.community.pocket.util.LocaleType;
import com.community.pocket.util.SPUtil;

import org.xutils.view.annotation.ContentView;
import org.xutils.view.annotation.ViewInject;

import java.util.List;

@ContentView(R.layout.garbage_sorting)
public class GarbageSortingFragment extends BaseFragment {

    @ViewInject(R.id.search_sorting)
    private AutoCompleteTextView searchSorting;

    //垃圾名
    @ViewInject(R.id.name)
    private TextView name;

    //垃圾分类
    @ViewInject(R.id.sorting)
    private TextView sorting;

    //垃圾图片
    @ViewInject(R.id.image)
    private ImageView image;

    private MyAutoCompleteTextView myAutoCompleteTextView;

    private GarbageSortingAdpter garbageSortingAdpter;

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        GarbageSortingViewModel viewModel = new ViewModelProvider(this, new ViewModelProvider.NewInstanceFactory()).get(GarbageSortingViewModel.class);

        myAutoCompleteTextView = new MyAutoCompleteTextView(searchSorting, viewModel);

        //监听垃圾分类检索状态
        viewModel.getGarbageSortingResponse().observe(getViewLifecycleOwner(), garbageSortingResponse -> {
            if (garbageSortingResponse == null) {
                return;
            }

            garbageSortingResponse.toast(getContext());

            if (garbageSortingResponse.getResult() == Response.Result.OK) {
                initSearch(garbageSortingResponse.getGarbageSortings());
            }
        });

        //添加监听触发搜索
        searchSorting.addTextChangedListener(new MyTextChange() {
            @Override
            public void afterTextChanged(Editable s) {
                if (garbageSortingAdpter != null && garbageSortingAdpter.containsKey(s.toString())) {
                    initView(garbageSortingAdpter.get(s.toString()));
                    return;
                }

                myAutoCompleteTextView.searchHandler();
            }
        });

    }

    /**
     * 初始化垃圾分类信息
     *
     * @param garbageSorting 垃圾分类
     */
    private void initView(GarbageSorting garbageSorting) {
        String n;
        if (SPUtil.getInstance(getContext()).getSelectLanguage() == LocaleType.en.getType()) {
            n = garbageSorting.getEnglishName();
            if (n == null) {
                n = garbageSorting.getName() + "(no english name)";
            }
        } else {
            n = garbageSorting.getName();
        }
        name.setText(n);
        int resId=R.string.unknow_error;
        switch (garbageSorting.getCategory()){
            case 1:resId=R.string.garbage_category_1;break;
            case 2:resId=R.string.garbage_category_2;break;
            case 4:resId=R.string.garbage_category_4;break;
            case 8:resId=R.string.garbage_category_8;break;
            case 16:resId=R.string.garbage_category_16;break;
            default:
        }
        sorting.setText(resId);
    }

    /**
     * 检索垃圾信息
     *
     * @param garbageSortings 垃圾分类信息
     */
    private void initSearch(List<GarbageSorting> garbageSortings) {
        // 设置适配器
        if (garbageSortingAdpter == null) {
            garbageSortingAdpter = new GarbageSortingAdpter(garbageSortings, getContext());
            // 将适配器与当前AutoCompleteTextView控件绑定
            searchSorting.setAdapter(garbageSortingAdpter);
        } else {
            garbageSortingAdpter.addAll(garbageSortings);
        }
        myAutoCompleteTextView.refreshDropList();
    }
}
