package com.community.pocket.ui.main.ui.share;

import android.content.Context;
import android.content.Intent;
import android.widget.Toast;

import com.community.pocket.R;
import com.community.pocket.data.model.LocalToken;
import com.community.pocket.ui.login.LoginActivity;
import com.community.pocket.util.AppDatabase;
import com.community.pocket.util.CustomMessage;

/**
 * 响应体父类
 *
 * @param <T>
 */
public abstract class Response<T extends CustomMessage> {
    //描述参数
    private Object[] args;

    //响应信息
    private T message;

    //操作结果
    private Result result;

    public T getMessage() {
        return message;
    }

    public void setMessage(T message) {
        this.message = message;
    }

    public Result getResult() {
        return result;
    }

    public void setResult(Result result) {
        this.result = result;
    }

    private Object[] getArgs() {
        return args;
    }

    public void setArgs(Object[] args) {
        this.args = args;
    }

    public void toast(Context context) {
        if (getMessage() != null) {
            Toast.makeText(context, context.getString(getMessage().getMsg(), getArgs()), getResult() == Result.OK ? Toast.LENGTH_SHORT : Toast.LENGTH_LONG).show();
            if (getMessage().getMsg() == R.string.invalid_token) {
                new Thread(() -> {
                    AppDatabase.getInstance(context).tokenDao().delete(LocalToken.getTokenInstance());
                    LocalToken.logout();
                    context.startActivity(new Intent(context, LoginActivity.class));
                }).start();
            }
        } else {
            Toast.makeText(context, R.string.unknow_error, Toast.LENGTH_LONG).show();
        }
    }

    public enum Result {
        OK,
        FAIL
    }
}
